# -*- coding: utf-8 -*-
"""
Analyse ring spectra results
"""

# %% Imports
from matplotlib import pyplot as plt

import numpy as np
from scipy import constants as const
from scipy import signal as sig
import os

from analysis import (load_results, plot_peaks, int_disp, calc_dispersion,
                      get_properties, plot_peakdata, filter_modes)


# %% Pyplot settings
plt.close('all')
import matplotlib as mpl
mpl.rcParams['axes.formatter.limits'] = -4,4


# %% Load results
subdir = os.getcwd() + '/ring36_TE/'
wls, p, p_un = load_results(subdir, plot=False, return_unfiltered=True)


# %% Settings
R = 150e-6  # Ring radius

fams = np.array([
    {   'name': 'TE00',  # Name of mode
        'wl0': 1550.534,  # Central peak in nm
        'fsr0': 1.2184,  # Central FSR in nm
        'a_over': 1e-5,  # Guess for normalised second order integrated dispersion
        'fitrange': 0.1/(wls[1]-wls[0])  # Passed to 'wlen' of scipy.signal.peak_prominences
    },
    {   'name': 'TE10',
        'wl0': 1550.458,
        'fsr0': 1.2045,
        'a_over': 0.5e-5,
        'fitrange': 0.1/(wls[1]-wls[0])
    },
    {   'name': 'm2',
        'wl0': 1550.1564,
        'fsr0': 1.1659,
        'a_over': -0.3e-5,
        'fitrange': 0.1/(wls[1]-wls[0])
    },
    {   'name': 'm3',
        'wl0': 1550.1064,
        'fsr0': 1.1650,
        'a_over': 0.5e-5,
        'fitrange': 0.1/(wls[1]-wls[0])
    },
    ])

# FSR as frequency
for fam in fams:
    fam['fsr_nu'] = const.c/((fam['wl0']*1e-9)**2) * fam['fsr0']*1e-9

# %% Find peaks
peaks, _ = sig.find_peaks(-p,
                       prominence=0.2, # Minimum prominence of 0.2
                       width=(2,80), # FWHM between 2 and 80 samples
                       plateau_size=(0,3)) # Maximum plateau of 3 points

# %% Plot peaks
# Linear data
p_lin = 10**(p/10)

# Plot peaks
#plot_peaks(wls, p, peaks)


# %% Filter based on integrated dispersion
# For each mode family
for n, fam in enumerate(fams):
    # Calculate and normalise integrated dispersion
    mu, Dint = int_disp(wls[peaks], fam['wl0'], fam['fsr0'])
    Dn = Dint/fam['fsr_nu'] # Normalise
    
    # Dispersion
    fam['coeff'], ind = filter_modes(wls[peaks], fam)
    fam['peaks'] = peaks[ind]
    
    # Plot and output dispersion
    calc_dispersion(fam['coeff'], fam['wl0'], fam['fsr_nu'], R,
                    doprint=False, store=fam)


# %% Calculate properties (without fitting)
for fam in fams:
    fam['prop'] = get_properties(wls, p_lin, fam)


# %% Plot integrated dispersion
# Integrated dispersion of all modes expanded around mode 0
plt.figure()
mu, Dint = int_disp(wls[peaks], fams[0]['wl0'], fams[0]['fsr0'])
plt.plot(mu, 1e-9*Dint, '.')
#plt.plot(mu, 1e-9*fams[0]['fsr_nu']*np.polyval(fams[0]['coeff'], mu), label='fit')


# Integrated dispersion, grouped by mode
for fam in fams:
    mu, Dint = int_disp(wls[fam['peaks']], fams[0]['wl0'], fams[0]['fsr0'])
    plt.plot(mu, 1e-9*Dint, '.', label=fam['name'])


# Plot settings
plt.title('Integrated dispersion')
plt.ylabel('$D_{int}/2\pi$ (GHz)')
plt.xlabel('μ')
ymax = 1e-9*fam['fsr_nu']/2
plt.ylim(-ymax, ymax)
plt.legend(loc=3)


# %% Plots
# Unfiltered signal
plt.figure(figsize=(6.2, 2.29))
plt.plot(wls, p_un)
for fam in fams:
    plt.plot(wls[fam['peaks']], p_un[fam['peaks']], 'x', markersize=4, label=fam['name'])
plt.title('Measured signal')
plt.ylabel('Throughput [dB]')
plt.xlabel('Wavelength [nm]')
plt.legend()
plt.xlim(1548, 1558)
plt.ylim(-35, -21)


# Propagation loss
# Calculate loss
for fam in fams:
    fam['prop']['loss'] = (fam['wl0']/(fam['fsr0'] * R * fam['prop']['Qint'])
                           * 10/np.log(10) * 1e-2)

# Histogram loss for first mode
fam = fams[0]
plt.figure()
plt.hist(fam['prop']['loss'], range=(0,1), bins=20)
plt.title('Propagation loss')
plt.ylabel('Counts')
plt.xlabel('loss [dB/cm]')


# To plot any other parameter (e.g. Q-factor)
"""
# Q-factor
plt.figure()
for fam in fams:
    plt.plot(wls[fam['peaks']], fam['prop']['Q'], '.', label=fam['name'])

plt.title('Q-factor')
plt.ylabel('Q-factor')
plt.xlabel('Wavelength [nm]')
plt.ylim(0, 1e6)
plt.legend()
"""

# Plot linear signal with peaks indicated
"""
plt.figure()
plt.plot(wls, p_lin)
for fam in fams:
    plt.plot(wls[fam['peaks']], p_lin[fam['peaks']], 'x', label=fam['name'])
    plot_peakdata(wls, p_lin, fam['peaks'], fam['prop'])

plt.title('Resonance')
plt.ylabel('Power [a.u.]')
plt.xlabel('Wavelength [nm]')
#plt.xlim(1549.35, 1549.75)
#plt.ylim(0, 2)
plt.legend()
"""
