# -*- coding: utf-8 -*-
"""
Tools for analysing ring spectra
"""

# %% Imports
from matplotlib import pyplot as plt
import numpy as np
import os
import re
import scipy.constants as const
from scipy import signal as sig
import warnings

# %% Helper functions
def sorted_alphanumeric(data):
    convert = lambda text: int(text) if text.isdigit() else text.lower()
    alphanum_key = lambda key: [ convert(c) for c in re.split('([0-9]+)', key) ] 
    return sorted(data, key=alphanum_key)

# %% Loading results
def load_results(subdir, plot=True, bgfitorder=2, return_unfiltered=False):
    wls = np.zeros(0)
    p = np.zeros(0)
    
    for fname in sorted_alphanumeric(os.listdir(subdir)):
        file = subdir + fname
        x,y = np.loadtxt(file, unpack=True)
        wls = np.hstack((wls, x))
        p = np.hstack((p, y))

    fit = np.polyfit(wls,p, bgfitorder)
    
    if plot: 
        # Plot signal
        plt.figure()
        plt.plot(wls, p)
        
        plt.plot(wls, np.polyval(fit, wls), color='k', linestyle='--')
        
        plt.title('Unfiltered signal')
        plt.ylabel('Output [dB]')
        plt.xlabel('Wavelength [nm]')
    
    p_filt = p - np.polyval(fit, wls)
    
    if return_unfiltered:
        return wls, p_filt, p
    else:
        return wls, p_filt

# %% Finding peaks
def plot_peaks(wls, p, peaks):
    # Method 1
    plt.figure()
    plt.plot(wls, p, label='signal')
    plt.plot(wls[peaks], p[peaks], 'x')
    
    plt.title('Peaks found using scipy')
    plt.xlabel('Wavelength [nm]')


# %% Peak properties
def get_properties(wls, p_lin, modeinfo):
    
    prop = dict()

    with warnings.catch_warnings():
        # Calc prominences, hide warning that some peaks have prominence of 0
        warnings.filterwarnings('ignore', message='some peaks have a prominence of 0')
        prop['prominence_data'] = sig.peak_prominences(-p_lin, modeinfo['peaks'],
                                                       wlen=modeinfo['fitrange'])
        
        
        # Remove peaks with prominence of 0
        false_peaks = np.where(prop['prominence_data'][0]==0)[0]
        prop['prominence_data'] = np.delete(prop['prominence_data'], false_peaks,
                                            axis=1).copy(order='C')
        modeinfo['peaks'] = np.delete(modeinfo['peaks'], false_peaks)
        
        # Save prominences separately for convenience
        prop['prominences'] = prop['prominence_data'][0]
    
    prop['width_data'] = sig.peak_widths(-p_lin, modeinfo['peaks'],
                                         wlen=modeinfo['fitrange'])
    prop['FWHM'] = prop['width_data'][0] * (wls[1]-wls[0])
    prop['Q'] = wls[modeinfo['peaks']]/prop['FWHM']
    
    Tmin = p_lin[modeinfo['peaks']]/(p_lin[modeinfo['peaks']] +
                                     prop['prominences'])
    prop['Qint'] = 2*prop['Q'] / (1+np.sqrt(Tmin))
    prop['Qext'] = 2*prop['Q'] / (1-np.sqrt(Tmin))
    prop['ER'] = prop['prominences']/(p_lin[modeinfo['peaks']] +
                                      prop['prominences'])
    
    return prop

def plot_peakdata(wls, signal, peaks, prop):
    plt.vlines(x = wls[peaks],
               ymin = signal[peaks],
               ymax = signal[peaks] + prop['prominences'],
               color = 'k',)
    plt.hlines(y = -prop['width_data'][1],
               xmin = wls[np.round(prop['width_data'][2]).astype(int)],
               xmax = wls[np.round(prop['width_data'][3]).astype(int)],
               color = 'k')


# %% Integrated dispersion
def int_disp(peakpos, wl0, fsr0):
    """
    Calculate integrated dispersion
    Input peak positions, wl and fsr of centre resonance, in nm"""
    peakpos = const.c/(peakpos*1e-9)
    res0 = const.c/(wl0*1e-9)
    fsr0 = const.c/((wl0*1e-9)**2) * fsr0*1e-9
    
    # Define mu and integrated dispersion
    mu = np.floor((peakpos - res0) / fsr0)
    Dint = (peakpos - res0) % fsr0
    # Centre integrated dispersion around 0
    Dint = (((Dint / fsr0) + 0.5)%1 - 0.5)*fsr0
    
    return mu, Dint

def calc_dispersion(coeff, wl0, fsr_nu, R, doprint=True, store=None):
    """
    Calculate dispersion based on fit of integrated dispersion    
    """
    # Calculate coefficients in MHz
    D = np.flip(coeff) * fsr_nu *1e-6
    D[2] = 2*D[2]
    D[3] = 3*2*D[3]
    
    # Calculate group index and dispersion
    ng = const.c/(fsr_nu * 2*np.pi*R)
    disp = 1/((wl0*1e-6)**2) * ng * (D[2])/((fsr_nu*1e-9)**2)
    
    # Print results
    res0 = const.c/(wl0*1e-9)
    if doprint:
        print(f'CF = {res0*1e-12:.2f} THz')
        print(f'FSR = {fsr_nu*1e-9:.2f} GHz')
        print(f'ng = {ng:.3f}')
        print(f'ω corr. = {D[0]:.2f} MHz')
        print(f'FSR corr. = {D[1]:.2f} MHz')
        print(f'D2/2pi = {D[2]:.2f} MHz')
        print(f'D3/2pi = {D[3]:.2f} MHz')
        print(f'D = {disp:.2f} ps/nm/km')
    
    if store is not None:
        info = {
            'CF' : f'{res0*1e-12:.2f} THz',
            'FSR' : f'{fsr_nu*1e-9:.2f} GHz',
            'ng' : f'{ng:.3f}',
            'ω corr.' : f'{D[0]:.2f} MHz',
            'FSR corr.' : f'{D[1]:.2f} MHz',
            'D2/2pi' : f'{D[2]:.2f} MHz',
            'D3/2pi' : f'{D[3]:.2f} MHz',
            'D' : f'{disp:.2f} ps/nm/km',
        }
        store['info'] = info


# %% Filtering based on integrated dispersion
def filter_modes(peakpos, modeinfo):
    wl0 = modeinfo['wl0']
    fsr0 = modeinfo['fsr0']
    a = modeinfo['a_over']
    
    # FSR in frequency
    fsr_nu = const.c/((wl0*1e-9)**2) * fsr0*1e-9    
    
    # Calculate and normalise integrated dispersion
    mu, Dint = int_disp(peakpos, wl0, fsr0)
    Dn = Dint/fsr_nu # Normalise

    # Filter on given second order dispersion
    y1 = a * mu**2
    ind = np.where((Dn > y1-0.02) & (Dn < y1+0.02))

    # Calculate 'True' dispersion, apply strict filter
    coeff = np.polyfit(mu[ind], Dn[ind], deg=3)
    y = np.polyval(coeff, mu)
    ind = np.where((Dn > y-0.01) & (Dn < y+0.01))
    
    return coeff, ind