# -*- coding: utf-8 -*-
"""
Load and plot OSA spectra
"""

from matplotlib import pyplot as plt
import numpy as np
from scipy.io import loadmat

plt.close('all')

import matplotlib as mpl
mpl.rcParams['lines.linewidth'] = 0.5


# %% Functions

def load_OSA_spectrum(file, cutoff=-60):
    """Load spectrum result from Optical Spectrum Analyser"""
    
    # Retrieve results
    rawdata = loadmat(file)
    
    # Convert results to sensible format
    info = dict()
    for name in rawdata['state'].dtype.names:
        if name not in ['wl', 'model', 'max']:
            info[name] = rawdata['state'][name].flatten()[0].flatten()[0]
    
    info['model'] = rawdata['state']['model'].flatten()[0][0]
    
    res = rawdata['data'].flatten()
    res[res<cutoff] = cutoff
    wls = np.linspace(info['start'], info['stop'], len(res))
    
    return wls, res, info

def plot_OSA_spectrum(ax, file, label=None):
    """Plot spectrum"""
    
    # Load spectrum 
    wls, res, info = load_OSA_spectrum(file)
    ax.plot(wls*1e9, res, label=label)
    if label is not None: ax.set_title(label)
    
    ax.set_xlabel('Wavelength [nm]')
    ax.set_xlim(1420, 1680)
    
    return wls, res, info

def calc_power(EDFA, split, through):
    """
    Calculate on-chip power
    
    Parameters:
        EDFA: Power set on EDFA in dBm
        split: Unpolarized power split by polarization beam combiner
        Through: Measured troughput of system (off-resonance)
    
    Note:
        Splitter has an insertion loss of 2.75 dB
        On-chip propagation loss is estimated at 0.45 dB
        A 50/50 splitter is used post-chip, thus measured throughput should be doubled
    
    """
    
    pre_chip = 10**((EDFA-2.75)/10)-split
    post_chip = 2*10**((through+0.45)/10)
    coupling = np.sqrt(post_chip/pre_chip)    
    #print(coupling)
    if coupling < 0.43:
        coupling = 0.48

    return coupling * pre_chip
    


# %% Plots for paper

# Make subplot
# See https://matplotlib.org/stable/tutorials/intermediate/constrainedlayout_guide.html
# Also subplots documentation

cm = 1/2.54

grid = {'wspace' : 0.0,
        'hspace' : 0.0}

fig, axs = plt.subplots(1, 4, figsize=(12*cm, 4*cm),
                        sharex=True, sharey=True, gridspec_kw=grid)

plot_OSA_spectrum(axs[0], './2021-03-12/ring11.rpt004.mat', f'{calc_power(26,9.8,13.3):.0f} mW')
plot_OSA_spectrum(axs[1], './2021-03-12/ring11.rpt011.mat', f'{calc_power(27.8,18,15):.0f} mW')
plot_OSA_spectrum(axs[2], './2021-03-12/ring11.rpt012.mat', f'{calc_power(28.5,12,15.7):.0f} mW')
plot_OSA_spectrum(axs[3], './2021-03-10/ring11.rpt012.mat', f'{calc_power(30,13.4,16.3):.0f} mW')

axs[0].set_ylabel('Power [dBm]')

fig.set_constrained_layout_pads(w_pad=0.05*cm, h_pad=0.05*cm, wspace=0, hspace=0)
